@@ -64,17 +64,18 @@ async def transform_python(
             message="Repository doesn't have a commit",
         )
 
-    gql_params = await prepare_graphql_params(
-        db=request.app.state.db, branch=branch_params.branch, at=branch_params.at, service=request.app.state.service
-    )
+    async with db.start_session(read_only=True) as dbs:
+        gql_params = await prepare_graphql_params(
+            db=dbs, branch=branch_params.branch, at=branch_params.at, service=request.app.state.service
+        )
 
-    result = await graphql(
-        schema=gql_params.schema,
-        source=query.query.value,
-        context_value=gql_params.context,
-        root_value=None,
-        variable_values=params,
-    )
+        result = await graphql(
+            schema=gql_params.schema,
+            source=query.query.value,
+            context_value=gql_params.context,
+            root_value=None,
+            variable_values=params,
+        )
 
     data = extract_data(query_name=query.name.value, result=result)
 
@@ -88,6 +89,7 @@ async def transform_python(
         branch=branch_params.branch.name,
         transform_location=f"{transform.file_path.value}::{transform.class_name.value}",
         timeout=transform.timeout.value,
+        convert_query_response=transform.convert_query_response.value or False,
         data=data,
     )
 
@@ -127,17 +129,18 @@ async def transform_jinja2(
             message="Repository doesn't have a commit",
         )
 
-    gql_params = await prepare_graphql_params(
-        db=request.app.state.db, branch=branch_params.branch, at=branch_params.at, service=request.app.state.service
-    )
+    async with db.start_session(read_only=True) as dbs:
+        gql_params = await prepare_graphql_params(
+            db=dbs, branch=branch_params.branch, at=branch_params.at, service=request.app.state.service
+        )
 
-    result = await graphql(
-        schema=gql_params.schema,
-        source=query.query.value,
-        context_value=gql_params.context,
-        root_value=None,
-        variable_values=params,
-    )
+        result = await graphql(
+            schema=gql_params.schema,
+            source=query.query.value,
+            context_value=gql_params.context,
+            root_value=None,
+            variable_values=params,
+        )
 
     data = extract_data(query_name=query.name.value, result=result)
 
