@@ -28,6 +28,9 @@ from typing_extensions import Self
 from infrahub import config, lock
 from infrahub.constants.database import DatabaseType, Neo4jRuntime
 from infrahub.core import registry
+from infrahub.core.constants import (
+    GLOBAL_BRANCH_NAME,
+)
 from infrahub.core.query import QueryType
 from infrahub.exceptions import DatabaseError
 from infrahub.log import get_logger
@@ -125,9 +128,14 @@ class DatabaseSchemaManager:
         return self.get_full(branch=branch, duplicate=duplicate)
 
     def get_schema_branch(self, name: str) -> SchemaBranch:
-        if name not in self._db._schemas:
-            return registry.schema.get_schema_branch(name=name)
-        return self._db._schemas[name]
+        """Return a schema branch object based on its name.
+
+        If the branch is the global one, the default branch will be returned.
+        """
+        branch_name = registry.default_branch if name == GLOBAL_BRANCH_NAME else name
+        if branch_name not in self._db._schemas:
+            return registry.schema.get_schema_branch(name=branch_name)
+        return self._db._schemas[branch_name]
 
 
 class InfrahubDatabase:
@@ -165,19 +173,6 @@ class InfrahubDatabase:
         else:
             self.db_type = config.SETTINGS.database.db_type
 
-    def __del__(self) -> None:
-        if not self._session or not self._is_session_local or self._session.closed():
-            return
-
-        try:
-            loop = asyncio.get_running_loop()
-        except RuntimeError:
-            loop = None
-        if loop and loop.is_running():
-            loop.create_task(self._session.close())
-        else:
-            asyncio.run(self._session.close())
-
     @property
     def is_session(self) -> bool:
         if self._mode == InfrahubDatabaseMode.SESSION:
