@@ -8,6 +8,7 @@ from graphql import GraphQLResolveInfo
 from infrahub import lock
 from infrahub.core.account import GlobalPermission
 from infrahub.core.branch import Branch
+from infrahub.core.branch.enums import BranchStatus
 from infrahub.core.constants import (
     CheckType,
     GlobalPermissions,
@@ -29,13 +30,14 @@ from infrahub.exceptions import BranchNotFoundError, PermissionDeniedError, Vali
 from infrahub.graphql.mutations.main import InfrahubMutationMixin
 from infrahub.graphql.types.enums import CheckType as GraphQLCheckType
 from infrahub.graphql.types.task import TaskInfo
-from infrahub.lock import InfrahubLock, build_object_lock_name
+from infrahub.lock import InfrahubLock
 from infrahub.proposed_change.approval_revoker import do_revoke_approvals_on_updated_pcs
 from infrahub.proposed_change.constants import ProposedChangeApprovalDecision, ProposedChangeState
 from infrahub.proposed_change.models import RequestProposedChangePipeline
 from infrahub.workers.dependencies import get_event_service
 from infrahub.workflows.catalogue import PROPOSED_CHANGE_MERGE, REQUEST_PROPOSED_CHANGE_PIPELINE
 
+from ...core.node.lock_utils import build_object_lock_name
 from .main import InfrahubMutationOptions
 
 if TYPE_CHECKING:
@@ -155,6 +157,11 @@ class InfrahubProposedChangeMutation(InfrahubMutationMixin, Mutation):
         if updated_state == ProposedChangeState.MERGED:
             if will_be_draft:
                 raise ValidationError("A draft proposed change is not allowed to be merged")
+
+            source_branch = await Branch.get_by_name(db=graphql_context.db, name=obj.source_branch.value)
+            if source_branch.status == BranchStatus.NEED_UPGRADE_REBASE:
+                raise ValidationError("The branch must be upgraded and rebased prior to merging the proposed change")
+
             data["state"]["value"] = ProposedChangeState.MERGING.value
 
         proposed_change, result = await super().mutate_update(
