@@ -98,7 +98,7 @@ class IPPrefixPoolGetResource(Mutation):
                 "id": resource.id,
                 "kind": resource.get_kind(),
                 "identifier": data.get("identifier", None),
-                "display_label": await resource.render_display_label(db=graphql_context.db),
+                "display_label": await resource.get_display_label(db=graphql_context.db),
                 "branch": graphql_context.branch.name,
             },
         }
@@ -144,7 +144,7 @@ class IPAddressPoolGetResource(Mutation):
                 "id": resource.id,
                 "kind": resource.get_kind(),
                 "identifier": data.get("identifier"),
-                "display_label": await resource.render_display_label(db=graphql_context.db),
+                "display_label": await resource.get_display_label(db=graphql_context.db),
                 "branch": graphql_context.branch.name,
             },
         }
