@@ -2,11 +2,12 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING, Any
 
-from graphene import Boolean, Field, String
+from graphene import Boolean, Field, Int, String
 
 from infrahub.core.branch import Branch
 from infrahub.core.constants import GLOBAL_BRANCH_NAME
 
+from .enums import InfrahubBranchStatus
 from .standard_node import InfrahubObjectType
 
 if TYPE_CHECKING:
@@ -19,6 +20,8 @@ class BranchType(InfrahubObjectType):
     description = String(required=False)
     origin_branch = String(required=False)
     branched_from = String(required=False)
+    status = InfrahubBranchStatus(required=True)
+    graph_version = Int(required=False)
     created_at = String(required=False)
     sync_with_git = Boolean(required=False)
     is_default = Boolean(required=False)
