@@ -89,8 +89,10 @@ class ProposedChangeArtifactDefinition(BaseModel):
     definition_id: str
     definition_name: str
     artifact_name: str
-    query_name: str
+    query_name: str  # Deprecated
+    query_id: str
     query_models: list[str]
+    query_payload: str = Field(..., description="GraphQL query")
     repository_id: str
     transform_kind: str
     template_path: str = Field(default="")
