@@ -1,12 +1,15 @@
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Awaitable, Callable
 
 from prefect import get_run_logger, task
 from prefect.automations import AutomationCore
 from prefect.cache_policies import NONE
-from prefect.client.orchestration import PrefectClient
+from prefect.client.orchestration import PrefectClient, get_client
 from prefect.client.schemas.filters import DeploymentFilter, DeploymentFilterName
 from prefect.events.schemas.automations import Automation
+from prefect.exceptions import PrefectHTTPStatusError
 
+from infrahub import lock
+from infrahub.database import InfrahubDatabase
 from infrahub.trigger.models import TriggerDefinition
 
 from .models import TriggerSetupReport, TriggerType
@@ -27,7 +30,29 @@ def compare_automations(target: AutomationCore, existing: Automation) -> bool:
     return target_dump == existing_dump
 
 
-@task(name="trigger-setup", task_run_name="Setup triggers", cache_policy=NONE)  # type: ignore[arg-type]
+@task(name="trigger-setup-specific", task_run_name="Setup triggers of a specific kind", cache_policy=NONE)  # type: ignore[arg-type]
+async def setup_triggers_specific(
+    gatherer: Callable[[InfrahubDatabase | None], Awaitable[list[TriggerDefinition]]],
+    trigger_type: TriggerType,
+    db: InfrahubDatabase | None = None,
+) -> TriggerSetupReport:
+    async with lock.registry.get(
+        name=f"configure-action-rules-{trigger_type.value}", namespace="trigger-rules", local=False
+    ):
+        if db:
+            async with db.start_session(read_only=True) as dbs:
+                triggers = await gatherer(dbs)
+        else:
+            triggers = await gatherer(db)
+        async with get_client(sync_client=False) as prefect_client:
+            return await setup_triggers(
+                client=prefect_client,
+                triggers=triggers,
+                trigger_type=trigger_type,
+            )  # type: ignore[misc]
+
+
+@task(name="trigger-setup", task_run_name="Setup triggers", cache_policy=NONE)
 async def setup_triggers(
     client: PrefectClient,
     triggers: list[TriggerDefinition],
@@ -59,7 +84,9 @@ async def setup_triggers(
     existing_automations: dict[str, Automation] = {}
     if trigger_type:
         existing_automations = {
-            item.name: item for item in await client.read_automations() if item.name.startswith(trigger_type.value)
+            item.name: item
+            for item in await client.read_automations()
+            if item.name.startswith(f"{trigger_type.value}::")
         }
     else:
         existing_automations = {item.name: item for item in await client.read_automations()}
@@ -109,8 +136,14 @@ async def setup_triggers(
             continue
 
         report.deleted.append(existing_automation)
-        await client.delete_automation(automation_id=existing_automation.id)
-        log.info(f"{item_to_delete} Deleted")
+        try:
+            await client.delete_automation(automation_id=existing_automation.id)
+            log.info(f"{item_to_delete} Deleted")
+        except PrefectHTTPStatusError as exc:
+            if exc.response.status_code == 404:
+                log.info(f"{item_to_delete} was already deleted")
+            else:
+                raise
 
     if trigger_type:
         log.info(
