@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import warnings
+from enum import Enum
 from typing import TYPE_CHECKING, Any, Literal, overload
 from urllib.parse import urlencode
 
@@ -14,6 +15,13 @@ if TYPE_CHECKING:
     from .client import InfrahubClient, InfrahubClientSync
 
 
+class BranchStatus(str, Enum):
+    OPEN = "OPEN"
+    NEED_REBASE = "NEED_REBASE"
+    NEED_UPGRADE_REBASE = "NEED_UPGRADE_REBASE"
+    DELETING = "DELETING"
+
+
 class BranchData(BaseModel):
     id: str
     name: str
@@ -21,6 +29,8 @@ class BranchData(BaseModel):
     sync_with_git: bool
     is_default: bool
     has_schema_changes: bool
+    graph_version: int | None = None
+    status: BranchStatus = BranchStatus.OPEN
     origin_branch: str | None = None
     branched_from: str
 
@@ -34,6 +44,8 @@ BRANCH_DATA = {
     "is_default": None,
     "sync_with_git": None,
     "has_schema_changes": None,
+    "graph_version": None,
+    "status": None,
 }
 
 BRANCH_DATA_FILTER = {"@filters": {"name": "$branch_name"}}
@@ -71,7 +83,7 @@ class InfraHubBranchManagerBase:
 
 
 class InfrahubBranchManager(InfraHubBranchManagerBase):
-    def __init__(self, client: InfrahubClient):
+    def __init__(self, client: InfrahubClient) -> None:
         self.client = client
 
     @overload
@@ -188,9 +200,7 @@ class InfrahubBranchManager(InfraHubBranchManagerBase):
         query = Query(name="GetAllBranch", query=QUERY_ALL_BRANCHES_DATA)
         data = await self.client.execute_graphql(query=query.render(), tracker="query-branch-all")
 
-        branches = {branch["name"]: BranchData(**branch) for branch in data["Branch"]}
-
-        return branches
+        return {branch["name"]: BranchData(**branch) for branch in data["Branch"]}
 
     async def get(self, branch_name: str) -> BranchData:
         query = Query(name="GetBranch", query=QUERY_ONE_BRANCH_DATA, variables={"branch_name": str})
@@ -223,16 +233,14 @@ class InfrahubBranchManager(InfraHubBranchManagerBase):
 
 
 class InfrahubBranchManagerSync(InfraHubBranchManagerBase):
-    def __init__(self, client: InfrahubClientSync):
+    def __init__(self, client: InfrahubClientSync) -> None:
         self.client = client
 
     def all(self) -> dict[str, BranchData]:
         query = Query(name="GetAllBranch", query=QUERY_ALL_BRANCHES_DATA)
         data = self.client.execute_graphql(query=query.render(), tracker="query-branch-all")
 
-        branches = {branch["name"]: BranchData(**branch) for branch in data["Branch"]}
-
-        return branches
+        return {branch["name"]: BranchData(**branch) for branch in data["Branch"]}
 
     def get(self, branch_name: str) -> BranchData:
         query = Query(name="GetBranch", query=QUERY_ONE_BRANCH_DATA, variables={"branch_name": str})
@@ -292,13 +300,14 @@ class InfrahubBranchManagerSync(InfraHubBranchManagerBase):
             },
         }
 
-        query = Mutation(mutation="BranchCreate", input_data=input_data, query=MUTATION_QUERY_DATA)
+        mutation_query = MUTATION_QUERY_TASK if background_execution else MUTATION_QUERY_DATA
+        query = Mutation(mutation="BranchCreate", input_data=input_data, query=mutation_query)
         response = self.client.execute_graphql(query=query.render(), tracker="mutation-branch-create")
 
         # Make sure server version is recent enough to support background execution, as previously
         # using background_execution=True had no effect.
         if background_execution and "task" in response["BranchCreate"]:
-            return BranchData(**response["BranchCreate"]["task"]["id"])
+            return response["BranchCreate"]["task"]["id"]
         return BranchData(**response["BranchCreate"]["object"])
 
     def delete(self, branch_name: str) -> bool:
