@@ -5,30 +5,32 @@ from typing import Any
 
 
 class Error(Exception):
-    def __init__(self, message: str | None = None):
+    def __init__(self, message: str | None = None) -> None:
         self.message = message
         super().__init__(self.message)
 
 
 class JsonDecodeError(Error):
-    def __init__(self, message: str | None = None, content: str | None = None, url: str | None = None):
+    def __init__(self, message: str | None = None, content: str | None = None, url: str | None = None) -> None:
         self.message = message
         self.content = content
         self.url = url
         if not self.message and self.url:
             self.message = f"Unable to decode response as JSON data from {self.url}"
+            if self.content:
+                self.message += f". Server response: {self.content}"
         super().__init__(self.message)
 
 
 class ServerNotReachableError(Error):
-    def __init__(self, address: str, message: str | None = None):
+    def __init__(self, address: str, message: str | None = None) -> None:
         self.address = address
         self.message = message or f"Unable to connect to '{address}'."
         super().__init__(self.message)
 
 
 class ServerNotResponsiveError(Error):
-    def __init__(self, url: str, timeout: int | None = None, message: str | None = None):
+    def __init__(self, url: str, timeout: int | None = None, message: str | None = None) -> None:
         self.url = url
         self.timeout = timeout
         self.message = message or f"Unable to read from '{url}'."
@@ -38,7 +40,7 @@ class ServerNotResponsiveError(Error):
 
 
 class GraphQLError(Error):
-    def __init__(self, errors: list[dict[str, Any]], query: str | None = None, variables: dict | None = None):
+    def __init__(self, errors: list[dict[str, Any]], query: str | None = None, variables: dict | None = None) -> None:
         self.query = query
         self.variables = variables
         self.errors = errors
@@ -47,21 +49,21 @@ class GraphQLError(Error):
 
 
 class BranchNotFoundError(Error):
-    def __init__(self, identifier: str, message: str | None = None):
+    def __init__(self, identifier: str, message: str | None = None) -> None:
         self.identifier = identifier
         self.message = message or f"Unable to find the branch '{identifier}' in the Database."
         super().__init__(self.message)
 
 
 class SchemaNotFoundError(Error):
-    def __init__(self, identifier: str, message: str | None = None):
+    def __init__(self, identifier: str, message: str | None = None) -> None:
         self.identifier = identifier
         self.message = message or f"Unable to find the schema '{identifier}'."
         super().__init__(self.message)
 
 
 class ModuleImportError(Error):
-    def __init__(self, message: str | None = None):
+    def __init__(self, message: str | None = None) -> None:
         self.message = message or "Unable to import the module"
         super().__init__(self.message)
 
@@ -73,7 +75,7 @@ class NodeNotFoundError(Error):
         message: str = "Unable to find the node in the database.",
         branch_name: str | None = None,
         node_type: str | None = None,
-    ):
+    ) -> None:
         self.node_type = node_type or "unknown"
         self.identifier = identifier
         self.branch_name = branch_name
@@ -95,25 +97,25 @@ class NodeInvalidError(NodeNotFoundError):
 class ResourceNotDefinedError(Error):
     """Raised when trying to access a resource that hasn't been defined."""
 
-    def __init__(self, message: str | None = None):
+    def __init__(self, message: str | None = None) -> None:
         self.message = message or "The requested resource was not found"
         super().__init__(self.message)
 
 
 class InfrahubCheckNotFoundError(Error):
-    def __init__(self, name: str, message: str | None = None):
+    def __init__(self, name: str, message: str | None = None) -> None:
         self.message = message or f"The requested InfrahubCheck '{name}' was not found."
         super().__init__(self.message)
 
 
 class InfrahubTransformNotFoundError(Error):
-    def __init__(self, name: str, message: str | None = None):
+    def __init__(self, name: str, message: str | None = None) -> None:
         self.message = message or f"The requested InfrahubTransform '{name}' was not found."
         super().__init__(self.message)
 
 
 class ValidationError(Error):
-    def __init__(self, identifier: str, message: str | None = None, messages: list[str] | None = None):
+    def __init__(self, identifier: str, message: str | None = None, messages: list[str] | None = None) -> None:
         self.identifier = identifier
         self.message = message
         self.messages = messages
@@ -128,7 +130,7 @@ class ValidationError(Error):
 
 
 class ObjectValidationError(Error):
-    def __init__(self, position: list[int | str], message: str):
+    def __init__(self, position: list[int | str], message: str) -> None:
         self.position = position
         self.message = message
         super().__init__(self.message)
@@ -138,13 +140,13 @@ class ObjectValidationError(Error):
 
 
 class AuthenticationError(Error):
-    def __init__(self, message: str | None = None):
+    def __init__(self, message: str | None = None) -> None:
         self.message = message or "Authentication Error, unable to execute the query."
         super().__init__(self.message)
 
 
 class URLNotFoundError(Error):
-    def __init__(self, url: str):
+    def __init__(self, url: str) -> None:
         self.message = f"`{url}` not found."
         super().__init__(self.message)
 
@@ -162,12 +164,12 @@ class InvalidResponseError(Error):
 
 
 class FileNotValidError(Error):
-    def __init__(self, name: str, message: str = ""):
+    def __init__(self, name: str, message: str = "") -> None:
         self.message = message or f"Cannot parse '{name}' content."
         super().__init__(self.message)
 
 
 class TimestampFormatError(Error):
-    def __init__(self, message: str | None = None):
+    def __init__(self, message: str | None = None) -> None:
         self.message = message or "Invalid timestamp format"
         super().__init__(self.message)
