@@ -15,7 +15,7 @@ if TYPE_CHECKING:
 class Attribute:
     """Represents an attribute of a Node, including its schema, value, and properties."""
 
-    def __init__(self, name: str, schema: AttributeSchemaAPI, data: Any | dict):
+    def __init__(self, name: str, schema: AttributeSchemaAPI, data: Any | dict) -> None:
         """
         Args:
             name (str): The name of the attribute.
@@ -76,6 +76,8 @@ class Attribute:
         variables: dict[str, Any] = {}
 
         if self.value is None:
+            if self._schema.optional and self.value_has_been_mutated:
+                data["value"] = None
             return data
 
         if isinstance(self.value, str):
