@@ -1,11 +1,14 @@
 from __future__ import annotations
 
+from collections import defaultdict
 from collections.abc import Iterable
 from typing import TYPE_CHECKING, Any
 
 from ..exceptions import (
+    Error,
     UninitializedError,
 )
+from ..types import Order
 from .constants import PROPERTIES_FLAG, PROPERTIES_OBJECT
 from .related_node import RelatedNode, RelatedNodeSync
 
@@ -18,7 +21,7 @@ if TYPE_CHECKING:
 class RelationshipManagerBase:
     """Base class for RelationshipManager and RelationshipManagerSync"""
 
-    def __init__(self, name: str, branch: str, schema: RelationshipSchemaAPI):
+    def __init__(self, name: str, branch: str, schema: RelationshipSchemaAPI) -> None:
         """
         Args:
             name (str): The name of the relationship.
@@ -104,7 +107,7 @@ class RelationshipManager(RelationshipManagerBase):
         branch: str,
         schema: RelationshipSchemaAPI,
         data: Any | dict,
-    ):
+    ) -> None:
         """
         Args:
             name (str): The name of the relationship.
@@ -156,8 +159,26 @@ class RelationshipManager(RelationshipManagerBase):
             self.peers = rm.peers
             self.initialized = True
 
+        ids_per_kind_map = defaultdict(list)
         for peer in self.peers:
-            await peer.fetch()  # type: ignore[misc]
+            if not peer.id or not peer.typename:
+                raise Error("Unable to fetch the peer, id and/or typename are not defined")
+            ids_per_kind_map[peer.typename].append(peer.id)
+
+        batch = await self.client.create_batch()
+        for kind, ids in ids_per_kind_map.items():
+            batch.add(
+                task=self.client.filters,
+                kind=kind,
+                ids=ids,
+                populate_store=True,
+                branch=self.branch,
+                parallel=True,
+                order=Order(disable=True),
+            )
+
+        async for _ in batch.execute():
+            pass
 
     def add(self, data: str | RelatedNode | dict) -> None:
         """Add a new peer to this relationship."""
@@ -209,7 +230,7 @@ class RelationshipManagerSync(RelationshipManagerBase):
         branch: str,
         schema: RelationshipSchemaAPI,
         data: Any | dict,
-    ):
+    ) -> None:
         """
         Args:
             name (str): The name of the relationship.
@@ -261,8 +282,26 @@ class RelationshipManagerSync(RelationshipManagerBase):
             self.peers = rm.peers
             self.initialized = True
 
+        ids_per_kind_map = defaultdict(list)
         for peer in self.peers:
-            peer.fetch()
+            if not peer.id or not peer.typename:
+                raise Error("Unable to fetch the peer, id and/or typename are not defined")
+            ids_per_kind_map[peer.typename].append(peer.id)
+
+        batch = self.client.create_batch()
+        for kind, ids in ids_per_kind_map.items():
+            batch.add(
+                task=self.client.filters,
+                kind=kind,
+                ids=ids,
+                populate_store=True,
+                branch=self.branch,
+                parallel=True,
+                order=Order(disable=True),
+            )
+
+        for _ in batch.execute():
+            pass
 
     def add(self, data: str | RelatedNodeSync | dict) -> None:
         """Add a new peer to this relationship."""
