@@ -7,7 +7,7 @@ from dulwich.repo import Repo
 
 
 class GitRepoManager:
-    def __init__(self, root_directory: str, branch: str = "main"):
+    def __init__(self, root_directory: str, branch: str = "main") -> None:
         self.root_directory = root_directory
         self.branch = branch
         self.git: Repo = self.initialize_repo()
@@ -29,5 +29,4 @@ class GitRepoManager:
 
     @property
     def active_branch(self) -> str | None:
-        active_branch = porcelain.active_branch(self.root_directory).decode("utf-8")
-        return active_branch
+        return porcelain.active_branch(self.root_directory).decode("utf-8")
