@@ -3,14 +3,22 @@ from __future__ import annotations
 import re
 import warnings
 from datetime import datetime, timezone
-from typing import Literal
+from typing import Literal, TypedDict
 
+from typing_extensions import NotRequired
 from whenever import Date, Instant, LocalDateTime, OffsetDateTime, Time, ZonedDateTime
 
 from .exceptions import TimestampFormatError
 
 UTC = timezone.utc  # Required for older versions of Python
 
+
+class SubstractParams(TypedDict):
+    seconds: NotRequired[float]
+    minutes: NotRequired[float]
+    hours: NotRequired[float]
+
+
 REGEX_MAPPING = {
     "seconds": r"(\d+)(s|sec|second|seconds)",
     "minutes": r"(\d+)(m|min|minute|minutes)",
@@ -21,7 +29,7 @@ REGEX_MAPPING = {
 class Timestamp:
     _obj: ZonedDateTime
 
-    def __init__(self, value: str | ZonedDateTime | Timestamp | None = None):
+    def __init__(self, value: str | ZonedDateTime | Timestamp | None = None) -> None:
         if value and isinstance(value, ZonedDateTime):
             self._obj = value
         elif value and isinstance(value, self.__class__):
@@ -43,8 +51,7 @@ class Timestamp:
     @classmethod
     def _parse_string(cls, value: str) -> ZonedDateTime:
         try:
-            zoned_date = ZonedDateTime.parse_common_iso(value)
-            return zoned_date
+            return ZonedDateTime.parse_common_iso(value)
         except ValueError:
             pass
 
@@ -73,14 +80,19 @@ class Timestamp:
         except ValueError:
             pass
 
-        params: dict[str, float] = {}
+        params: SubstractParams = {}
         for key, regex in REGEX_MAPPING.items():
             match = re.search(regex, value)
             if match:
-                params[key] = float(match.group(1))
+                if key == "seconds":
+                    params["seconds"] = float(match.group(1))
+                elif key == "minutes":
+                    params["minutes"] = float(match.group(1))
+                elif key == "hours":
+                    params["hours"] = float(match.group(1))
 
         if params:
-            return ZonedDateTime.now("UTC").subtract(**params)  # type: ignore[call-overload]
+            return ZonedDateTime.now("UTC").subtract(**params)
 
         raise TimestampFormatError(f"Invalid time format for {value}")
 
