@@ -17,7 +17,7 @@ INFRAHUB_TRANSFORM_VARIABLE_TO_IMPORT = "INFRAHUB_TRANSFORMS"
 class InfrahubTransform(InfrahubOperation):
     name: str | None = None
     query: str
-    timeout: int = 10
+    timeout: int = 60
 
     def __init__(
         self,
@@ -27,7 +27,7 @@ class InfrahubTransform(InfrahubOperation):
         branch: str = "",
         root_directory: str = "",
         server_url: str = "",
-    ):
+    ) -> None:
         super().__init__(
             client=client,
             infrahub_node=infrahub_node,
